@@ -1,5 +1,6 @@
 import numpy as np
 from geosoup.common import Handler, Opt, Sublist
+from geosoup.exceptions import ImageProcessingError, ObjectNotFound
 import warnings
 import json
 from osgeo import gdal, gdal_array, ogr, osr, gdalconst
@@ -77,7 +78,7 @@ class Raster(object):
         self.transform = transform
         self.crs_string = crs_string
         self.name = name
-        self.dtype = dtype
+        self.dtype = gdal.GDT_Float32
         self.metadict = metadict
         self.nodatavalue = None
         self.tile_grid = list()
@@ -129,18 +130,20 @@ class Raster(object):
             self.metadict = Raster.get_raster_metadict(file_ptr=fileptr)
 
         else:
-            raise ValueError('No datasource found')
+            raise ImageProcessingError('No datasource found')
 
         if band_order is None:
             band_order = list(range(fileptr.RasterCount))
         elif max(band_order) >= fileptr.RasterCount:
-            raise ValueError('Band indices must be smaller than number of available bands')
+            raise ImageProcessingError('Band indices must be smaller than number of available bands')
 
         self.shape = [fileptr.RasterCount if not get_array else len(band_order),
                       fileptr.RasterYSize,
                       fileptr.RasterXSize]
 
-        self.dtype = fileptr.GetRasterBand(1).DataType
+        if self.dtype is not None:
+            self.dtype = fileptr.GetRasterBand(1).DataType
+
         self.nodatavalue = fileptr.GetRasterBand(1).GetNoDataValue()
 
         # if get_array flag is true
@@ -257,23 +260,15 @@ class Raster(object):
         if self.array is None:
             self.read_array()
 
-        if nbands == 1:
-            fileptr.GetRasterBand(1).WriteArray(self.array, 0, 0)
-            fileptr.GetRasterBand(1).SetDescription(self.bnames[0])
+        for i in range(0, nbands):
+            fileptr.GetRasterBand(i + 1).WriteArray(self.array[i, :, :], 0, 0)
+            fileptr.GetRasterBand(i + 1).SetDescription(self.bnames[i])
 
             if self.nodatavalue is not None:
-                fileptr.GetRasterBand(1).SetNoDataValue(self.nodatavalue)
+                fileptr.GetRasterBand(i + 1).SetNoDataValue(self.nodatavalue)
             if verbose:
-                Opt.cprint('Writing band: ' + self.bnames[0])
-        else:
-            for i in range(0, nbands):
-                fileptr.GetRasterBand(i + 1).WriteArray(self.array[i, :, :], 0, 0)
-                fileptr.GetRasterBand(i + 1).SetDescription(self.bnames[i])
+                Opt.cprint('Writing band: ' + self.bnames[i])
 
-                if self.nodatavalue is not None:
-                    fileptr.GetRasterBand(i + 1).SetNoDataValue(self.nodatavalue)
-                if verbose:
-                    Opt.cprint('Writing band: ' + self.bnames[i])
         if driver == 'MEM':
             if verbose:
                 Opt.cprint('File written to memory!')
@@ -347,7 +342,7 @@ class Raster(object):
                    offsets=None,
                    band_order=None):
         """
-        read raster array with offsets
+        Method to read raster array with offsets and a specific band order
         :param offsets: tuple or list - (xoffset, yoffset, xcount, ycount)
         :param band_order: order of bands to read
         """
@@ -480,7 +475,7 @@ class Raster(object):
 
             return any([not x for x in truth_about_empty_bands])
         else:
-            raise ValueError("File does not exist.")
+            raise ObjectNotFound("File does not exist.")
 
     def make_tiles(self,
                    tile_size_x,
@@ -577,7 +572,8 @@ class Raster(object):
             else:
                 raise AttributeError("Metadata dictionary does not exist.")
         else:
-            raise ValueError("Tile size {}x{} is larger than original raster {}x{}.".format(tile_size_y,
+            raise ImageProcessingError("Tile size "
+                                       "{}x{} is larger than original raster {}x{}.".format(tile_size_y,
                                                                                             tile_size_x,
                                                                                             self.shape[1],
                                                                                             self.shape[2]))
@@ -596,13 +592,13 @@ class Raster(object):
                 # open raster
                 img_pointer = gdal.Open(file_name)
             else:
-                raise ValueError("File does not exist.")
+                raise ObjectNotFound("File does not exist.")
 
         elif file_ptr is not None:
             img_pointer = file_ptr
 
         else:
-            raise ValueError("File or pointer not found")
+            raise ObjectNotFound("File or pointer not found")
 
         # get tiepoint, pixel size, pixel rotation
         geometadata = img_pointer.GetGeoTransform()
@@ -655,7 +651,7 @@ class Raster(object):
                 scale_approx=False):
 
         """
-        Method to change the raster data type
+        Method to rescale raster or to change the raster data type
         :param out_type: Out data type.
                          Valid options: byte, int, long, float, double
                                         int8, int16, int32, int64,
@@ -885,6 +881,25 @@ class Raster(object):
                 out_datasource = None
                 return geom_wkt
 
+    def centroid(self,
+                 use_nodatavalue=False):
+        """
+        Method to return centroid of the Raster object based on its boundary
+        :param use_nodatavalue: Boolean flag to calculate the centroid
+                                based on valid pixel boundary
+        """
+        bounds = self.get_bounds(use_nodatavalue=use_nodatavalue,
+                                 xy_coordinates=False)
+
+        if use_nodatavalue:
+            bounds_geom = ogr.CreateGeometryFromWkt(bounds)
+            centroid_geom = bounds_geom.Centroid()
+            return centroid_geom.GetPoint()[0:2]
+
+        else:
+            bounds = self.get_bounds()
+            return float(bounds[0] + bounds[1])/2.0, float(bounds[2] + bounds[3])/2.0
+
     def get_pixel_bounds(self,
                          bound_coords=None,
                          coords_type='pixel'):
@@ -913,7 +928,7 @@ class Raster(object):
                     int(coords_locations[:, 1].min()), \
                     int(coords_locations[:, 1].max())
             else:
-                raise ValueError("Unknown coordinate types")
+                raise ImageProcessingError("Unknown coordinate types")
 
             if xmin < 0:
                 xmin = 0
@@ -925,9 +940,9 @@ class Raster(object):
                 ymax = self.shape[1]
 
             if xmin >= xmax:
-                raise ValueError("Image x-size should be greater than 0")
+                raise ImageProcessingError("Image x-size should be greater than 0")
             if ymin >= ymax:
-                raise ValueError("Image y-size should be greater than 0")
+                raise ImageProcessingError("Image y-size should be greater than 0")
         else:
             xmin, xmax, ymin, ymax = 0, self.shape[2], 0, self.shape[1]
 
@@ -952,12 +967,14 @@ class Raster(object):
         if not self.init:
             self.initialize()
 
+        self.tile_grid = []
+
         # convert to the number of pixels in the buffer region
         if tile_buffer is not None:
             buf_size_x = np.ceil(float(tile_buffer)/abs(float(self.transform[1])))
             buf_size_y = np.ceil(float(tile_buffer)/abs(float(self.transform[5])))
         else:
-            buf_size_x = buf_size_y = None
+            buf_size_x, buf_size_y = None, None
 
         xmin, xmax, ymin, ymax = self.get_pixel_bounds(bound_coords,
                                                        coords_type)
@@ -1025,7 +1042,7 @@ class Raster(object):
             bands = list(range(1, self.shape[0] + 1))
 
         if block_coords is None:
-            raise ValueError("Block coords needed to retrieve tile")
+            raise ImageProcessingError("Block coords needed to retrieve tile")
         else:
             upperleft_x, upperleft_y, tile_rows, tile_cols = block_coords
 
@@ -1213,6 +1230,11 @@ class Raster(object):
                                      Used only if the pass_pixel_coords flag is set to true.
                                      Returns the pixel centers of each pixel if set as true
                                      else returns top left corners
+                        min_pixels: Minimum number of pixels for reducer to calculate a value.
+                                    If the number of pixels are less than this, then an empty list is returned
+                        replace: If the pixel values in Raster.array object should be replaced with
+                                 the reduced value? This will only be computed of min_pixels condition is
+                                 satisfied and will change the Raster array. Use with caution.
                         reducer: Valid keywords: 'mean','median','max',
                                                  'min', 'percentile_xx' where xx is percentile from 1-99
 
@@ -1253,6 +1275,16 @@ class Raster(object):
         else:
             reducer = None
 
+        if 'min_pixels' in kwargs:
+            min_pixels = kwargs['min_pixels']
+        else:
+            min_pixels = None
+
+        if 'replace' in kwargs:
+            replace = kwargs['replace']
+        else:
+            replace = False
+
         # define band order
         if band_order is None:
             band_order = list(range(0, self.shape[0]))
@@ -1277,7 +1309,7 @@ class Raster(object):
         # prepare dict struct
         out_geom_extract = dict()
         for internal_id, _ in id_geom_dict.items():
-            out_geom_extract[internal_id] = {'values': [], 'coordinates': []}
+            out_geom_extract[internal_id] = {'values': [], 'coordinates': [], 'xyz_loc': []}
 
         # list of sample ids
         for tile in self.tile_grid:
@@ -1381,13 +1413,33 @@ class Raster(object):
                         out_geom_extract[geom_id]['values'] += list(tile_arr[band_order, x, y].tolist()
                                                                     for x, y in pixel_xy_loc)
 
+                        out_geom_extract[geom_id]['xyz_loc'] += list([[band_index] + xy_loc]
+                                                                     for xy_loc in pixel_xy_loc
+                                                                     for band_index in band_order)
+
             if reducer is not None:
                 warned = False
                 for geom_id, geom_dict in out_geom_extract.items():
                     try:
-                        geom_dict['values'] = Sublist.reduce(geom_dict['values'],
-                                                             method=reducer,
-                                                             axis=0).tolist()
+                        n_pixels = len(geom_dict['values'])
+                        if min_pixels is None:
+                            min_pixels = n_pixels
+
+                        if n_pixels >= min_pixels:
+                            geom_dict['values'] = Sublist.reduce(geom_dict['values'],
+                                                                 method=reducer,
+                                                                 axis=0).tolist()
+
+                            if replace:
+                                pixel_loc = (np.array(geom_dict['xyz_loc'])[:, 0],
+                                             np.array(geom_dict['xyz_loc'])[:, 1],
+                                             np.array(geom_dict['xyz_loc'])[:, 2])
+
+                                self.array[pixel_loc] = geom_dict['values']
+
+                        else:
+                            geom_dict['values'] = []
+
                         geom_dict['coordinates'] = Sublist.reduce(geom_dict['coordinates'],
                                                                   method=reducer,
                                                                   axis=0).tolist()
@@ -1543,7 +1595,7 @@ class Raster(object):
             elif out_proj4 is not None:
                 res = spref.ImportFromProj4(out_proj4)
             else:
-                raise ValueError("Output Spatial reference not provided")
+                res = spref.ImportFromWkt(self.crs_string)
 
         warp_dict['srcSRS'] = self.crs_string
         warp_dict['dstSRS'] = spref.ExportToWkt()
@@ -1860,7 +1912,7 @@ class MultiRaster:
                 pctl = int(composite_type.split('_')[1])
                 temp_arr = np.apply_along_axis(lambda x: np.percentile(x[x != lras.nodatavalue], pctl), 0, tile_arr)
             else:
-                raise ValueError('Unknown composite option')
+                raise ImageProcessingError('Unknown composite option')
 
             # update output array with tile composite
             out_arr[_y: (_y+_rows), _x: (_x+_cols)] = temp_arr
